/**
 * Copyright (c) 2022, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AccessControlConstants, AccessControlUtils, Show } from "@wso2is/access-control";
import { resolveUserDisplayName } from "@wso2is/core/helpers";
import { IdentifiableComponentInterface, ProfileInfoInterface, RouteInterface } from "@wso2is/core/models";
import { RouteUtils } from "@wso2is/core/utils";
import { GenericIcon, Heading, LinkButton, PageLayout, Text } from "@wso2is/react-components";
import axios from "axios";
import cloneDeep from "lodash-es/cloneDeep";
import React, { FunctionComponent, ReactElement, useCallback, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch, useSelector } from "react-redux";
import { Button, Card, Grid, Popup, Label, Placeholder } from "semantic-ui-react";
import { CardExpandedNavigationButton } from "./card-expanded-navigation-button";
import { LoginPlaygroundWizard } from "./login-playground-wizard";
import { MinimalAppCreateWizard } from "../../../../features/applications";
import {
    getApplicationDetails,
    getApplicationList,
    getInboundProtocolConfig
} from "../../../../features/applications/api";
import {
    ApplicationManagementConstants
} from "../../../../features/applications/constants";
import {
    ApplicationListInterface,
    ApplicationListItemInterface,
    ApplicationTemplateListItemInterface
} from "../../../../features/applications/models";
import {
    ApplicationManagementUtils
} from "../../../../features/applications/utils";
import {
    AppConstants,
    AppState,
    ConfigReducerStateInterface,
    EventPublisher,
    FeatureConfigInterface,
    history,
    setActiveView,
    setDeveloperVisibility,
    setManageVisibility
} from "../../../../features/core";
import { getAdminViewRoutes, getDeveloperViewRoutes } from "../../../../features/core/configs";
import { UserListInterface, getUsersList } from "../../../../features/users";
import { ReactComponent as SupportHeadsetIcon } from "../../../assets/images/icons/support-headset-icon.svg";
import { AppViewExtensionTypes, commonConfig } from "../../../configs";
import { CONSUMER_USERSTORE } from "../../users/constants";
import { DynamicApplicationContextCard } from "../components/dynamic-application-context-card";
import { getGettingStartedCardIllustrations } from "../configs";
import { getTryItClientId } from "../../application/utils/try-it-utils";

/**
 * Proptypes for the overview page component.
 */
interface AdvanceUserViewInterface extends IdentifiableComponentInterface {
    onApplicationCreate: () => void;
 }

/**
 * Overview page.
 *
 * @param {GettingStartedPageInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
const AdvanceUserView: FunctionComponent<AdvanceUserViewInterface> = (
    props: AdvanceUserViewInterface
): ReactElement => {

    const {
        ["data-componentid"]: componentId,
        onApplicationCreate
    } = props;

    const { t } = useTranslation();

    const dispatch = useDispatch();

    const profileInfo: ProfileInfoInterface = useSelector((state: AppState) => state.profile.profileInfo);
    const isProfileInfoLoading: boolean = useSelector((state: AppState) => state.loaders.isProfileInfoRequestLoading);
    const activeView: string = useSelector((state: AppState) => state.global.activeView);
    const docSiteURL: string = useSelector((state: AppState) => state.config.deployment.docSiteURL);
    const config: ConfigReducerStateInterface = useSelector((state: AppState) => state.config);
    const allowedScopes: string = useSelector((state: AppState) => state?.auth?.allowedScopes);
    const featureConfig: FeatureConfigInterface = useSelector((state: AppState) => state.config.ui.features);
    const tenantDomain: string = useSelector((state: AppState) => state.auth.tenantDomain);

    const [ isConsumersAvailable, setIsConsumersAvailable ] = useState<boolean>(undefined);
    const [ isApplicationsAvailable, setIsApplicationsAvailable ] = useState<boolean>(undefined);
    const [
        isConsumerUsersFetchRequestLoading,
        setIsConsumerUsersFetchRequestLoading
    ] = useState<boolean>(undefined);
    const [
        isApplicationsFetchRequestLoading,
        setIsApplicationsFetchRequestLoading
    ] = useState<boolean>(undefined);

    const [ showWizard, setShowWizard ] = useState<boolean>(false);
    const [ selectedTemplate, setSelectedTemplate ] = useState<ApplicationTemplateListItemInterface>(null);
    const [ applications, setApplications ] = useState<ApplicationListItemInterface[]>([]);
    const [ isPlaygroundExist, setisPlaygroundExist ] = React.useState(false);
    const [ showWizardLogin, setShowWizardLogin ] = useState<boolean>(false);
    const [ isPlaygroundLoading, setisPlaygroundLoading ] = useState<boolean>(false);
    const [ inboundProtocolConfig, setInboundProtocolConfig ] = useState<any>(undefined);

    const eventPublisher: EventPublisher = EventPublisher.getInstance();

    let response: ApplicationListInterface = null;

    /**
     *  Retreiving the URL for the Asgardeo Login Playground
     */
    const asgardeoLoginPlaygroundURL = window["AppUtils"].getConfig().extensions.asgardeoTryItURL;

    const handleTryLoginClick = () => {
        if(isPlaygroundExist){
            window.open(asgardeoLoginPlaygroundURL+"?client_id="+getTryItClientId(tenantDomain)+"&org="+tenantDomain);
        }else{
            eventPublisher.publish("application-quick-start-click-add-user");
            setShowWizardLogin(true);
        }
    };

    const customizeLoginFlow = useCallback(async (data: string) => {
        response = await getApplicationList(null, null, "name eq " + data);
        history.push(AppConstants.getPaths()
            .get("APPLICATION_EDIT").replace(":id", response.applications[0].id+"#tab=2") );
    }, []);

    /**
     * Checking whether the playground application already exist or not
     */
    const isPlaygroundApplicationExists = async () => {
        setisPlaygroundLoading(true);
        response = await getApplicationList(null, null, "name eq " + "Asgardeo Try It");
        if (response?.applications?.length > 0 ) {
            const applicationDetailPromises = [];
            let protocolConfigs: any = {};

            response.applications.forEach((application) => {
                applicationDetailPromises.push(getApplicationDetails(application.id));
            });

            axios.all(applicationDetailPromises)
                .then(axios.spread((...responses) => {
                    getInboundProtocolConfig(responses[0].id,
                        ApplicationManagementUtils.mapProtocolTypeToName(responses[0].inboundProtocols[0].type))
                        .then((response) => {
                            protocolConfigs = {
                                ...protocolConfigs,[responses[0].inboundProtocols[0].type]:response
                            };
                            setInboundProtocolConfig(protocolConfigs);
                        })
                        .finally(() =>{
                            if (protocolConfigs?.oauth2?.clientId === getTryItClientId(tenantDomain)) {
                                setisPlaygroundExist(true);
                                setisPlaygroundLoading(false);
                            }else{
                                setisPlaygroundExist(false);
                                setisPlaygroundLoading(false);
                            }
                        });
                }));
        }else {
            setisPlaygroundExist(false);
            setisPlaygroundLoading(false);
        }
    };

    /**
     * Make sure `QUICKSTART` tab is highlighed when this page is in use.
     */
    useEffect(() => {
        if (activeView === AppViewExtensionTypes.QUICKSTART) {
            return;
        }

        dispatch(setActiveView(AppViewExtensionTypes.QUICKSTART));
    }, []);

    useEffect(() => {
        const manageRoutes: RouteInterface[] = RouteUtils.sanitizeForUI(cloneDeep(getAdminViewRoutes()));
        const developRoutes: RouteInterface[] = RouteUtils.sanitizeForUI(cloneDeep(getDeveloperViewRoutes()));

        const tab: string = AccessControlUtils.getDisabledTab(
            manageRoutes, developRoutes, allowedScopes, featureConfig, commonConfig.checkForUIResourceScopes);

        if (tab === "MANAGE") {
            dispatch(setManageVisibility(false));
        } else if (tab === "DEVELOP") {
            dispatch(setDeveloperVisibility(false));
        } else if (tab === "BOTH") {
            dispatch(setManageVisibility(false));
            dispatch(setDeveloperVisibility(false));

            history.push({
                pathname: AppConstants.getPaths().get("UNAUTHORIZED"),
                search: "?error=" + AppConstants.LOGIN_ERRORS.get("ACCESS_DENIED")
            });

        } else {
            dispatch(setManageVisibility(true));
            dispatch(setDeveloperVisibility(true));
        }
    }, []);

    useEffect(() => {

        setIsConsumerUsersFetchRequestLoading(true);

        getUsersList(null, null, null, "id", CONSUMER_USERSTORE)
            .then((response: UserListInterface) => {
                setIsConsumersAvailable(response.totalResults > 0);
            })
            .catch(() => {
                // Add debug logs here one a logger is added.
                // Tracked here https://github.com/wso2/product-is/issues/11650.
            })
            .finally(() => {
                setIsConsumerUsersFetchRequestLoading(false);
            });
    }, []);

    useEffect(() => {

        setIsApplicationsFetchRequestLoading(true);

        getApplicationList(null, null, null)
            .then((response: ApplicationListInterface) => {
                setIsApplicationsAvailable(response.totalResults > 0);
                setApplications(response.applications);
            })
            .catch(() => {
                // Add debug logs here one a logger is added.
                // Tracked here https://github.com/wso2/product-is/issues/11650.
            })
            .finally(() => {
                setIsApplicationsFetchRequestLoading(false);
            });
    }, []);

    /**
      * Monitor `profileInfo.id` and publish the event to avoid an event without `UUID`.
      */
    useEffect(() => {

        if (!profileInfo?.id) {
            return;
        }

        // TODO: Move this to the `extensions/configs/common`.
        // Tracked here https://github.com/wso2-enterprise/asgardeo-product/issues/7742#issuecomment-939960128.
        eventPublisher.publish("console-click-getting-started-menu-item");
    }, [ profileInfo?.id ]);

    useEffect(() => {

        isPlaygroundApplicationExists();
    }, [ ]);

    return (
        <PageLayout
            padded={ false }
            contentTopMargin={ false }
            data-componentid={ `${componentId}-layout` }
            className="getting-started-page"
        >
            <div className="advance-user-view-cards-wrapper">
                <div className="greeting">
                    <Heading
                        ellipsis
                        compact
                        as="h1"
                        bold="500"
                        data-testid="welcome-greeting-header"
                        data-componentid="welcome-greeting-header"
                        data-suppress=""
                    >
                        {
                            isProfileInfoLoading
                                ? (
                                    <Placeholder
                                        data-testid="welcome-greeting-placeholder"
                                        data-componentid="welcome-greeting-placeholder"
                                    >
                                        <Placeholder.Header>
                                            <Placeholder.Line length="very long"/>
                                        </Placeholder.Header>
                                    </Placeholder>
                                )
                                : t("extensions:common.quickStart.greeting.heading", {
                                    username: resolveUserDisplayName(profileInfo)
                                })
                        }
                    </Heading>
                </div>
                <Grid stackable>
                    <Grid.Row columns={ 2 }>
                        <Show when={ AccessControlConstants.APPLICATION_READ }>
                            <Grid.Column
                                stretched
                                mobile={ 16 }
                                tablet={ 16 }
                                computer={ 6 }
                                largeScreen={ 6 }
                                widescreen={ 6 }
                            >
                                <DynamicApplicationContextCard
                                    applications={ applications }
                                    isApplicationsAvailable={ isApplicationsAvailable }
                                    isApplicationsFetchRequestLoading={ isApplicationsFetchRequestLoading }
                                    onTemplateSelected={ (template) => {
                                        setSelectedTemplate(template);
                                        setShowWizard(true);
                                    } }
                                />
                            </Grid.Column>
                        </Show>
                        <Grid.Column
                            stretched
                            mobile={ 16 }
                            tablet={ 16 }
                            computer={ 10 }
                            largeScreen={ 10 }
                            widescreen={ 10 }>
                            <Grid>
                                <Grid.Row columns={ 2 }>
                                    <Show when={ AccessControlConstants.USER_READ }>
                                        <Grid.Column
                                            stretched
                                            mobile={ 16 }
                                            tablet={ 16 }
                                            computer={ 8 }
                                            largeScreen={ 8 }
                                            widescreen={ 8 }>
                                            <Card
                                                fluid
                                                className="basic-card no-hover getting-started-card manage-users-card"
                                            >
                                                <Card.Content extra className="description-container">
                                                    <div className="card-heading mb-1">
                                                        <Heading as="h2">
                                                            Manage users
                                                        </Heading>
                                                    </div>
                                                    <Text muted>
                                                        Manage users in your organization who will access the  applications
                                                    </Text>
                                                </Card.Content>
                                                <Card.Content style={ { borderTop: "none" } } className="illustration-container">
                                                    <GenericIcon
                                                        relaxed="very"
                                                        size="small"
                                                        transparent
                                                        className="onboard-users-animated-illustration mb-5"
                                                        icon={ getGettingStartedCardIllustrations().onboardUsers }
                                                    />
                                                </Card.Content>
                                                <Card.Content className="action-container" extra>
                                                    <CardExpandedNavigationButton
                                                        data-testid="getting-started-page-add-user-button"
                                                        data-componentid="getting-started-page-add-user-button"
                                                        onClick={ () => {
                                                            eventPublisher.publish("console-getting-started-add-users-path");
                                                            history.push(AppConstants.getPaths().get("USERS"));
                                                        } }
                                                        text="View users"
                                                        icon="angle right"
                                                        iconPlacement="right"
                                                        className="primary-action-button"
                                                    />
                                                </Card.Content>
                                            </Card>
                                        </Grid.Column>
                                    </Show>
                                    <Show when={ AccessControlConstants.IDP_READ }>
                                        <Grid.Column
                                            stretched
                                            mobile={ 16 }
                                            tablet={ 16 }
                                            computer={ 8 }
                                            largeScreen={ 8 }
                                            widescreen={ 8 }>
                                            <Card
                                                fluid
                                                className="basic-card no-hover getting-started-card social-connections-card"
                                            >
                                                <Card.Content className="description-container">
                                                    <div className="card-heading mb-1">
                                                        <Heading as="h2">
                                                            Enhance app login
                                                        </Heading>
                                                    </div>
                                                    <Text muted>
                                                        { t("extensions:common.quickStart.sections.addSocialLogin.description") }
                                                    </Text>
                                                </Card.Content>
                                                <Card.Content style={ { borderTop: "none" } } className="illustration-container">
                                                    <GenericIcon
                                                        relaxed="very"
                                                        size="small"
                                                        transparent
                                                        className="social-connections-animated-illustration mb-5"
                                                        icon={ getGettingStartedCardIllustrations().setupSocialConnections }
                                                    />
                                                </Card.Content>
                                                <Card.Content extra className="action-container">
                                                    <CardExpandedNavigationButton
                                                        data-testid="develop-getting-started-page-add-social-login"
                                                        data-componentid="develop-getting-started-page-add-social-login"
                                                        onClick={ () => {
                                                            eventPublisher.publish("console-getting-started-add-social-connection-path");
                                                            history.push({
                                                                pathname: AppConstants.getPaths().get("CONNECTIONS")
                                                            });
                                                        } }
                                                        text="Set up social connections"
                                                        icon="angle right"
                                                        iconPlacement="right"
                                                        className="primary-action-button"
                                                    />
                                                </Card.Content>
                                            </Card>
                                        </Grid.Column>
                                    </Show>
                                </Grid.Row>
                                <Grid.Row>
                                    <Grid.Column>
                                        {
                                            featureConfig.tryIt?.enabled &&
                                            (
                                                <Card
                                                    fluid
                                                    className="basic-card no-hover"
                                                >
                                                    <Card.Content>
                                                        <Grid>
                                                            <Grid.Row>
                                                                <Grid.Column width={ 3 } verticalAlign="middle" className="pr-0">
                                                                    <GenericIcon
                                                                        style={ {
                                                                            width: "111.26px",
                                                                            height: "91.3px"
                                                                        } }
                                                                        floated="left"
                                                                        transparent
                                                                        icon={ getGettingStartedCardIllustrations().tryItApplication }
                                                                    />
                                                                </Grid.Column>
                                                                <Grid.Column width={ 10 } className="pl-0">
                                                                    <div className="card-heading pt-3 mb-1">
                                                                        <Heading as="h2">
                                                                            Try login with the Try It app
                                                                            <Label
                                                                                color="purple"
                                                                                size="mini"
                                                                                className="feature-status-label new ml-2"
                                                                            >
                                                                                NEW
                                                                            </Label>
                                                                        </Heading>
                                                                    </div>
                                                                    <Text muted>
                                                                        Use the hosted sample application to try out with login
                                                                        flow changes and check the look and feel of it.
                                                                    </Text>
                                                                </Grid.Column>
                                                                <Grid.Column verticalAlign="middle"  width={ 3 }>
                                                                    {
                                                                        isPlaygroundLoading
                                                                            ?  <LinkButton loading={ true } compact inverted />
                                                                            :  (
                                                                                <>
                                                                                    <Popup
                                                                                        position="top center"
                                                                                        trigger={
                                                                                            <Button
                                                                                                floated="right"
                                                                                                data-testid="develop-getting-started-page-try-it"
                                                                                                data-componentid="develop-getting-started-page-try-it"
                                                                                                onClick={ handleTryLoginClick }
                                                                                                icon="angle right"
                                                                                                iconPlacement="right"
                                                                                                className="primary-action-button mr-3"
                                                                                                size="large"
                                                                                            />
                                                                                        }
                                                                                        content="Try It"
                                                                                        inverted
                                                                                    />
                                                                                    {
                                                                                        isPlaygroundExist && (
                                                                                            <Popup
                                                                                                position="top center"
                                                                                                trigger={
                                                                                                    <Button
                                                                                                        floated="right"
                                                                                                        data-testid="develop-getting-started-page-cutomize-try-it"
                                                                                                        data-componentid="develop-getting-started-page-cutomize-try-it"
                                                                                                        onClick={ () => {
                                                                                                            customizeLoginFlow("Asgardeo Try It");
                                                                                                        } }
                                                                                                        icon="setting"
                                                                                                        iconPlacement="right"
                                                                                                        size="large"
                                                                                                        className="primary-action-button"
                                                                                                    />
                                                                                                }
                                                                                                content="Customize Login Flow"
                                                                                                inverted
                                                                                            />
                                                                                        )
                                                                                    }
                                                                                </>
                                                                            )
                                                                    }
                                                                </Grid.Column>
                                                            </Grid.Row>
                                                        </Grid>
                                                    </Card.Content>
                                                </Card>
                                            )
                                        }

                                    </Grid.Column>
                                </Grid.Row>
                            </Grid>
                        </Grid.Column>
                    </Grid.Row>
                </Grid>
                <div className="community-link-container" data-componentid="community-link-section">
                    <div className="contact-icon-container">
                        <GenericIcon
                            transparent
                            icon={ SupportHeadsetIcon }
                            size="mini"
                            fill="default"
                            spaced="right"
                        />
                    </div>
                    <div>
                        <Heading as="h5" compact>Help & Support</Heading>
                        <Text compact muted>For assistance, contact
                            { " " }
                            <a
                                href="mailto:asgardeo-help@wso2.com"
                                data-componentid="community-link-mail-trigger"
                            >
                                asgardeo-help@wso2.com
                            </a>
                        </Text>
                    </div>
                </div>
                { showWizard && (
                    <MinimalAppCreateWizard
                        title={ selectedTemplate?.name }
                        subTitle={ selectedTemplate?.description }
                        closeWizard={ (): void => {
                            setShowWizard(false);
                            setSelectedTemplate(undefined);
                        } }
                        template={ selectedTemplate }
                        showHelpPanel={ true }
                        subTemplates={ selectedTemplate?.subTemplates }
                        subTemplatesSectionTitle={ selectedTemplate?.subTemplatesSectionTitle }
                        addProtocol={ false }
                        templateLoadingStrategy={
                            config.ui.applicationTemplateLoadingStrategy
                            ?? ApplicationManagementConstants.DEFAULT_APP_TEMPLATE_LOADING_STRATEGY
                        }
                    />
                ) }
                {
                    showWizardLogin && (
                        <LoginPlaygroundWizard
                            data-componentId="login-playground-wizard-modal"
                            closeWizard={ () => setShowWizardLogin(false) }
                            applicationName="Asgardeo Login Playground"
                            onApplicationCreate={ () => {
                                setisPlaygroundExist(true);
                                onApplicationCreate();
                            } }
                        />
                    )
                }
            </div>
        </PageLayout>
    );
};
 
/**
  * Default props for the component.
  */
AdvanceUserView.defaultProps = {
    "data-componentid": "getting-started-page"
};
 
/**
  * A default export was added to support React.lazy.
  * TODO: Change this to a named export once react starts supporting named exports for code splitting.
  * @see {@link https://reactjs.org/docs/code-splitting.html#reactlazy}
  */
export default AdvanceUserView;
